"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.renderColorscheme = void 0;
const handlebars_1 = __importDefault(require("handlebars"));
const hexterm_1 = __importDefault(require("hexterm"));
const crash_1 = require("./crash");
const util_1 = require("./util");
const legacy_ui_1 = require("./legacy-ui");
function renderColorscheme(config, project) {
    const palette = project.palettes[config.palette];
    if (!palette) {
        crash_1.crash('Colorscheme palette does not exist', {
            colorscheme: config.name,
            palette: config.palette
        });
    }
    const syntax = project.syntax;
    const c = parseSyntaxColors(syntax, palette);
    const render = handlebars_1.default.compile(project.mustaches.colorscheme);
    const info = {
        name: config.name,
        description: config.description,
        url: project.config.url,
        author: project.config.author,
        license: project.config.license,
        background: config.background,
        estiloVersion: util_1.estiloVersion
    };
    const term = parseTermColors(project.terminalSyntax, palette);
    return render({ c, info, term });
}
exports.renderColorscheme = renderColorscheme;
function parseTermColors(termSyntax, palette) {
    const values = {};
    Object.keys(termSyntax).forEach(prop => {
        const colorName = termSyntax[prop];
        const value = palette.colors[colorName];
        if (!value) {
            crash_1.crash('Missing terminal color', {
                colorName,
                property: prop,
                palette: palette.filepath
            });
        }
        values[prop] = value.hex;
    });
    return values;
}
function parseSyntaxColors(syntax, palette) {
    const values = {};
    syntax.forEach(rule => {
        const [fgColor, bgColor, ui, curlColor] = rule.rule.split(/\s+/);
        const filepath = rule.filepath;
        if (fgColor.startsWith('@')) {
            values[rule.name] = {
                link: fgColor.slice(1)
            };
        }
        else {
            values[rule.name] = {
                fore: getColorCode(fgColor, palette, filepath),
                back: getColorCode(bgColor, palette, filepath),
                ui: getUI(ui),
                guisp: getCurlColor(curlColor, palette, filepath)
            };
        }
    });
    return values;
}
function getColorCode(color, palette, filepath) {
    // return false if empty color
    if (color === '.')
        return false;
    // return false if color is `NONE`
    if (!color || color === '-')
        return { hex: 'NONE', xterm: 'NONE' };
    // return custom color if colorname is in palette
    const colorcodes = palette.colors[color];
    if (colorcodes)
        return colorcodes;
    // return direct hex color
    if (util_1.isHexColor(color)) {
        const finalcolor = color.startsWith('#') ? color : color.slice(1);
        return {
            hex: finalcolor,
            xterm: hexterm_1.default(color).toString()
        };
    }
    // not valid color
    crash_1.crash('Color does not exist', { filepath, color });
}
function getUI(ui) {
    // no defined gui
    if (ui === '.')
        return false;
    if (!ui)
        return 'NONE';
    // 'NONE' or empty value
    if (ui === 'NONE')
        return 'NONE';
    if (legacy_ui_1.isLegacyUi(ui))
        return legacy_ui_1.parseUi(ui);
    // validate(ui)
    return ui;
}
function getCurlColor(color, palette, filepath) {
    const curlParsed = getColorCode(color, palette, filepath);
    let curlColor;
    if (!curlParsed || curlParsed.hex === 'NONE') {
        curlColor = false;
    }
    else {
        curlColor = curlParsed;
    }
    return curlColor;
}
